#Suppression des objets en mmoire
rm(list=ls(all=TRUE))

#Les fonctions et les donnes utilises sont ranges dans les repertoires suivants
ProjectionFonctionsR="C:\\Documents and Settings\\akamega\\Mes documents\\1389 - Livre\\Projection\\Projection R Fonctions"
CalibrageDonnees="C:\\Documents and Settings\\akamega\\Mes documents\\1389 - Livre\\Calibrage\\Calibrage Donnees"

#Chargement du package utilis
require(matrixStats)

##PARTIE 0 : Initialisation des paramtres (valeurs initiales et rsultat des calibrages)

NS=10000 # nombre de simulations
T=30 # maturit des projections
ForceLong=0.451 # force de rappel du taux long
MoyenneLong=0.029 # moyenne de long terme du taux long
VolatiliteLong=0.023 # volatilit du processus des taux long
TauxLongt=0.031 # taux d'intert long  l'instant t (ici  l'instant 0)
TauxCourtt=0.015 # taux d'intert court  l'instant t (ici  l'instant 0)
ForceCourt=0.397 # force de rappel du taux court
VolatiliteCourt=0.024 # volatilit du processus des taux court
ForceI=0.261 # force de rappel de l'inflation
MoyenneI=0.051 # moyenne de long terme de l'inflation
VolatiliteI=0.026 # volatilit de l'inflation
It=0.015 # inlfation  l'instant t (ici  l'instant 0)
Immot=0.081 # rendement de l'immobilier  l'instant t (ici  l'instant 0)
ForceImmo=0.152 # force de rappel de l'immobilier
MoyenneImmo=0.094 # moyenne de long terme du redt immobilier
VolatiliteImmo=0.037 # Volatilit de l'immobilier
SigmaExcesAct=0.175 # volatilit de l'exces de rendement des actions
MuExcesAct=0.033 # valeur moyenne de l'exces de rendement des actions
M=1 # maturit des ZC

##PARTIE 1 : Dtermination de la matrice de corrlation des rsidus

#Chargement des donnes
setwd(CalibrageDonnees)
Tx_Inflation<-scan("Tx_Inflation.txt")
Rdt_Immobilier<-scan("Rdt_Immobilier.txt")
Tx_reel_long<-scan("Tx_reel_long.txt")
Tx_reel_court<-scan("Tx_reel_court.txt")
Rdt_exces_actions<-scan("Rdt_exces_actions.txt")

#Dfinition du nombre d'observation dans le fichier en entre
Nobs<-length(Tx_Inflation)
#Dfinition des sries
Tx_Inflation_tplusun=Tx_Inflation[2:Nobs]
Tx_Inflation_t=Tx_Inflation[1:(Nobs-1)]
Rdt_Immobilier_tplusun=Rdt_Immobilier[2:Nobs]
Rdt_Immobilier_t=Rdt_Immobilier[1:(Nobs-1)]
Tx_reel_long_tplusun=Tx_reel_long[2:Nobs]
Tx_reel_long_t=Tx_reel_long[1:(Nobs-1)]
Tx_reel_court_t=Tx_reel_court[1:(Nobs-1)]
Tx_reel_court_tplusun=Tx_reel_court[2:Nobs]

#Rgression par les MCO
reg_Tx_Inflation=lm(Tx_Inflation_tplusun~Tx_Inflation_t)
reg_Rdt_Immobilier=lm(Rdt_Immobilier_tplusun~Rdt_Immobilier_t)
reg_Tx_reel_long=lm(Tx_reel_long_tplusun~Tx_reel_long_t)

#Calcul de la diffrence entre les taux de LT estims et les taux de CT observs et ajustement de la taille du vecteur
Chap_Tx_reel_long_tplusun=coef(reg_Tx_reel_long)[1]+coef(reg_Tx_reel_long)[2]*Tx_reel_long_t
Diff_ChapLT_CT=Chap_Tx_reel_long_tplusun-Tx_reel_court_tplusun
Diff_ChapLT_CT=Diff_ChapLT_CT[1:(Nobs-2)]
#Calcul de la diffrence entre les taux de CT observs en t+1 et ceux observs en t
Diff_CT=Tx_reel_court_tplusun-Tx_reel_court_t
Diff_CT=Diff_CT[2:(Nobs-1)]
#Rgression par les MCO (suite)
reg_DiffTx_reel_court=lm(Diff_CT~Diff_ChapLT_CT-1)

#Dtermination des rsidus
Resid_Tx_Inflation=resid(reg_Tx_Inflation)/(sd(resid(reg_Tx_Inflation))*sqrt((Nobs-2)/(Nobs-3)))
Resid_Tx_Inflation=Resid_Tx_Inflation[2:(Nobs-1)]
Resid_Rdt_Immobilier=resid(reg_Rdt_Immobilier)/(sd(resid(reg_Rdt_Immobilier))*sqrt((Nobs-2)/(Nobs-3)))
Resid_Rdt_Immobilier=Resid_Rdt_Immobilier[2:(Nobs-1)]
Resid_Tx_reel_long=resid(reg_Tx_reel_long)/(sd(resid(reg_Tx_reel_long))*sqrt((Nobs-2)/(Nobs-3)))
Resid_Tx_reel_long=Resid_Tx_reel_long[2:(Nobs-1)]
Resid_Tx_reel_court=resid(reg_DiffTx_reel_court)/(sd(resid(reg_DiffTx_reel_court))*sqrt(((Nobs-1)-2)/((Nobs-1)-3)))
Resid_Rdt_exces_actions=(Rdt_exces_actions-(mean(Rdt_exces_actions)-(sd(Rdt_exces_actions)^2)/2))/(sd(Rdt_exces_actions)*sqrt(((Nobs+1)-2)/((Nobs+1)-3)))
Resid_Rdt_exces_actions=Resid_Rdt_exces_actions[3:(Nobs)]

#Rangement des rsidus dans un tableau
Tableau_ResidAhlgrim=data.frame(Resid_Tx_Inflation,Resid_Rdt_Immobilier,Resid_Tx_reel_long,Resid_Tx_reel_court,Resid_Rdt_exces_actions)

#Calcul de la matrice de corrlation
Corr_ResidAhlgrim=cor(Tableau_ResidAhlgrim)

##PARTIE 2 : Projection des rsidus en tenant compte des corrlations

#Factorisation de la Cholesky de la matrice de corrlation (matrice triangulaire infrieure)
Mat_Chol=t(chol(Corr_ResidAhlgrim))
#Calcul du nombre d'indice d'actifs dans la matrice de corrlation
Nbcorr=sqrt(length(Corr_ResidAhlgrim))
#Simulation matrice de lois N(0,1)  3 dimensions (nb correlations, nb annees projection, nb simulations)
Mat_norm=array(rnorm(Nbcorr*NS*T),dim=c(Nbcorr,NS,T))
#Initialisation  0 de la matrice  3 dimensions des rsidus
Mat_res=array(0,dim=c(Nbcorr,NS,T),dimnames = list(c("Inflation", "Immobilier", "Tx_long","Tx_court","Exces_actions"),1:NS,1:T))
#Construction de la matrice  3 dimensions avec les rsidus projets
for(i in 1:Nbcorr){
  for(n in 1:NS){
    for(k in 1:T){  
    Mat_res[i,n,k]= sum(Mat_Chol[i,]*Mat_norm[,n,k]) 
    }
  }
}

#Backtesting sur la matrice de corrlation
Corr_Mat_res=array(0,dim=c(NS,Nbcorr,Nbcorr))
for(n in 1:NS){
  Corr_Mat_res[n,,]=cor(t(Mat_res[,n,1:T])) 
}
Test_corr_Mat_res=array(0,dim=c(Nbcorr,Nbcorr),dimnames = list(c("Inflation", "Immobilier", "Tx_long","Tx_court","Exces_actions"),c("Inflation", "Immobilier", "Tx_long","Tx_court","Exces_actions")))
Test_corr_Mat_res[,]=apply(Corr_Mat_res,c(2,3),mean)

##PARTIE 3 : Projection du rendement des actions

# Chargement des fonctions utilises
source(paste(ProjectionFonctionsR,"\\F_Vacisek1.R",sep=""))
source(paste(ProjectionFonctionsR,"\\F_Blackscholes.R",sep=""))
source(paste(ProjectionFonctionsR,"\\F_Vacisek2.R",sep=""))

#Initialisation des actifs pour le calcul du rendement des actions
TauxLong<-matrix(0,nrow=NS,ncol=(T+1));TauxLong[,1]=TauxLongt
TauxCourt<-matrix(0,nrow=NS,ncol=(T+1));TauxCourt[,1]=TauxCourtt
Inflation<-matrix(0,nrow=NS,ncol=(T+1));Inflation[,1]=It
TauxNominalCourt<-matrix(0,nrow=NS,ncol=(T+1))
ExcesRdt<-matrix(0,nrow=NS,ncol=(T+1))
RdtAction<-matrix(0,nrow=NS,ncol=(T+1))

#Simulations des actifs pour les actions pour toutes les priodes de projection et rinitialisation des paramtres
for(k in 1:T){
  #Simulations
  TauxLong[,k+1]<-Vacisek2(NS,TauxLongt,ForceLong,MoyenneLong,VolatiliteLong,TauxCourtt,ForceCourt,VolatiliteCourt,Mat_res["Tx_long",,k],Mat_res["Tx_court",,k],ProjectionFonctionsR)[,"TauxLong"]
  TauxCourt[,k+1]<-Vacisek2(NS,TauxLongt,ForceLong,MoyenneLong,VolatiliteLong,TauxCourtt,ForceCourt,VolatiliteCourt,Mat_res["Tx_long",,k],Mat_res["Tx_court",,k],ProjectionFonctionsR)[,"TauxCourt"]
  Inflation[,k+1]<-Vacisek1(NS,It,ForceI,MoyenneI,VolatiliteI,Mat_res["Inflation",,k])
  TauxNominalCourt[,k+1]<-(1+TauxCourt[,k+1])*(1+Inflation[,k+1])-1
  ExcesRdt[,k+1]<-Blackscholes(NS,SigmaExcesAct,MuExcesAct,Mat_res["Exces_actions",,k])
  RdtAction[,k+1]=TauxNominalCourt[,k+1]+ExcesRdt[,k+1]
  #Rinitialisation
  TauxLongt<-TauxLong[,k+1]
  TauxCourtt<-TauxCourt[,k+1]
  It<-Inflation[,k+1]
}
#Prsentation des rsultats des projections (moyenne des simulations) pour le rendement des actions
plot(((2007+2):(2007+(T))),colMeans(TauxNominalCourt[,2:(T)]),type="l",ylim=c(0,0.15),lwd=2,lty=5,pch=5,main="Taux nominal court (Ahlgrim / Friggit)",xlab="Anne de projection",ylab="Taux moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(TauxNominalCourt[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(TauxNominalCourt[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

plot(((2007+2):(2007+(T))),colMeans(ExcesRdt[,2:(T)]),type="l",ylim=c(-0.25,0.35),lwd=2,lty=5,pch=5,main="Excs de rendement des actions (Ahlgrim / Friggit)",xlab="Anne de projection",ylab="Excs de rendement moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(ExcesRdt[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(ExcesRdt[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

plot(((2007+2):(2007+(T))),colMeans(RdtAction[,2:(T)]),type="l",ylim=c(-0.2,0.4),lwd=2,lty=5,pch=5,main="Action (Ahlgrim / Friggit)",xlab="Anne de projection",ylab="Rendement moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(RdtAction[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(RdtAction[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

##PARTIE 4 : Projection du prix des ZC rels

#Prsentation des rsultats des projections des taux (moyenne des simulations) pour ZC rels
plot(((2007+2):(2007+(T))),colMeans(TauxLong[,2:(T)]),type="l",ylim=c(-0.05,0.1),lwd=2,lty=5,pch=5,main="Taux rel long (Ahlgrim / Friggit)",xlab="Anne de projection",ylab="Taux moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(TauxLong[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(TauxLong[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

plot(((2007+2):(2007+(T))),colMeans(TauxCourt[,2:(T)]),type="l",ylim=c(-0.05,0.1),lwd=2,lty=5,pch=5,main="Taux rel court (Ahlgrim / Friggit)",xlab="Anne de projection",ylab="Taux moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(TauxCourt[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(TauxCourt[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

#Formule explicite du calcul des prix des ZC rels
B1=array(0,dim=c(T))
B2=array(0,dim=c(T))
A=array(0,dim=c(T))
ZC_reel=matrix(0,nrow=NS,ncol=(T+1))
for(k in 1:T){
  B2[k]=((1-exp(-ForceCourt*M))/ForceCourt)
  B1[k]=(ForceCourt/(ForceCourt-ForceLong))*(((1-exp(-ForceLong*(M)))/ForceLong)-((1-exp(-ForceCourt*(M)))/ForceCourt))
  A[k]=(B2[k]-(M))*(MoyenneLong-(VolatiliteCourt^2/(2*ForceCourt^2)))+B1[k]*MoyenneLong-(VolatiliteCourt^2*B2[k]^2)/(4*ForceCourt)+(VolatiliteLong^2/2)*((M)/ForceLong^2-2*((B1[k]+B2[k])/ForceLong^2)+(1-exp(-2*ForceCourt*(M)))/(2*ForceCourt*(ForceCourt-ForceLong)^2)-(2*ForceCourt*(1-exp((-ForceLong-ForceCourt)*(M))))/(ForceLong*(ForceCourt-ForceLong)^2*(ForceCourt+ForceLong))+ (ForceCourt^2*(1-exp(-2*ForceLong*(M)))/(ForceLong^2*(ForceCourt-ForceLong)^2*2*ForceLong)))
  for(n in 1:NS){
    ZC_reel[n,k+1]=exp(A[k]-B2[k]*TauxCourt[n,k+1]-B1[k]*TauxLong[n,k+1])
  }
}  
plot(((2007+2):(2007+(T))),colMeans(ZC_reel[,2:(T)]),type="l",ylim=c(0.9,1.05),lwd=2,lty=5,pch=5,main="ZC rel (Maturit 1 an) (Ahlgrim / Friggit)",xlab="Anne de projection",ylab="Prix moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(ZC_reel[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(ZC_reel[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

##PARTIE 5 : Projection du prix des ZC nominaux

#Prsentation de l'inflation projete (moyenne des simulations) en vue du calcul du prix des ZC en nominal
plot(((2007+2):(2007+(T))),colMeans(Inflation[,2:(T)]),type="l",ylim=c(-0.05,0.1),lwd=2,lty=5,pch=5,main="Inflation (Ahlgrim / Friggit)",xlab="Anne de projection",ylab="Taux moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(Inflation[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(Inflation[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

#Formule explicite du calcul des prix des ZC nominaux
B=array(0,dim=c(T))
A=array(0,dim=c(T))
ZC_infl=matrix(0,nrow=NS,ncol=(T+1))
ZC_nom=matrix(0,nrow=NS,ncol=(T+1))
for(k in 1:T){
    B[k]=((1-exp(-ForceI*(M)))/ForceI)
    A[k]=exp(((B[k]-M)*(ForceI^2*MoyenneI-VolatiliteI^2/2)/ForceI^2)-(VolatiliteI^2*B[k]^2/(4*ForceI)))
  for(n in 1:NS){    
    ZC_infl[n,k+1]=A[k]*exp(-B[k]*Inflation[n,k+1]) 
    ZC_nom[n,k+1]=ZC_reel[n,k+1]*ZC_infl[n,k+1]
    }
}
plot(((2007+2):(2007+(T))),colMeans(ZC_nom[,2:(T)]),type="l",ylim=c(0.85,1),lwd=2,lty=5,pch=5,main="ZC nominal (Maturit 1 an) (Ahlgrim / Friggit)",xlab="Anne de projection",ylab="Prix moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(ZC_nom[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(ZC_nom[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

##PARTIE 6 : Projection du rendement de l'immobilier

#Initialisation des actifs pour le calcul du rendement des actions
RdtImmo<-matrix(0,nrow=NS,ncol=(T+1));RdtImmo[,1]=Immot

#Simulations des actifs pour les actions pour toutes les priodes de projection et rinitialisation des paramtres
for(k in 1:T){
  #Simulations
  RdtImmo[,k+1]<-Vacisek1(NS,Immot,ForceImmo,MoyenneImmo,VolatiliteImmo,Mat_res["Immobilier",,k])
  #Rinitialisation
  Immot<-RdtImmo[,k+1]
}
#Prsentation des rsultats des projections (moyenne des simulations) pour le rendement de l'immobilier
plot(((2007+2):(2007+(T))),colMeans(RdtImmo[,2:(T)]),type="l",ylim=c(0,0.2),lwd=2,lty=5,pch=5,main="Immobilier (Ahlgrim / Friggit)",xlab="Anne de projection",ylab="Rendement moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(RdtImmo[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(RdtImmo[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)
