#Suppression des objets en mmoire
rm(list=ls(all=TRUE))

#Les fonctions et les donnes utilises sont ranges dans les repertoires suivants
ProjectionFonctionsR="C:\\Documents and Settings\\akamega\\Mes documents\\1389 - Livre\\Projection\\Projection R Fonctions"
CalibrageDonnees="C:\\Documents and Settings\\akamega\\Mes documents\\1389 - Livre\\Calibrage\\Calibrage Donnees"

#Chargement du package utilis
require(matrixStats)

##PARTIE 0 : Initialisation des paramtres (valeurs initiales et rsultat des calibrages)

NS=10000 # nombre de simulations
T=30 # maturit des projections
TauxCourtt=0.015 # taux d'intert court  l'instant t (ici  l'instant 0)
ForceCourt=0.346 # force de rappel du taux court
MoyenneCourt=0.018 # moyenne de long terme du taux long
VolatiliteCourt=0.024 # volatilit du processus des taux court
ForceI=0.261 # force de rappel de l'inflation ralise
MoyenneI=0.051 # moyenne de long terme de l'inflation ralise
VolatiliteI=0.026 # volatilit de l'inflation ralise
It=0.015 # inlfation  l'instant t (ici  l'instant 0)
SigmaAct=0.171 # volatilit du rendement des actions
MuAct=0.101 # valeur moyenne du rendement des actions
SigmaIPC=0.037 # volatilit de l'inflation attendue (IPC)
MuIPC=MoyenneI # valeur moyenne de l'inflation attendue (IPC)
M=1 # maturit des ZC
Xi_S=0.0039 # influence du bruit des actions sur l'IPC
Xi_r=0.0075 # influence du bruit des taux d'intrt sur l'IPC
Xi_I=0.0296 # influence du bruit de l'inflation attendue sur l'IPC
Xi_u=0.0281 # influence du bruit de l'inflation inattendue sur l'IPC
Lambda_S=0.168 # prime de risque unitaire des actions
Lambda_r=-0.209 # prime de risque unitaire des taux d'intrt
Lambda_I=-0.105 # prime de risque unitaire de l'inflation attendue
Lambda_u=0.028 # prime de risque unitaire de l'inflation inattendue
Phi_S=-0.162 # influence du bruit des actions sur le facteur d'actualisation
Phi_r=0.552 # influence du bruit des taux d'intrt sur le facteur d'actualisation
Phi_I=0.464 # influence du bruit des taux d'inflation sur le facteur d'actualisation

##PARTIE 1 : Dtermination de la matrice de corrlation des rsidus

#Chargement des donnes
setwd(CalibrageDonnees)
Tx_Inflation<-scan("Tx_Inflation.txt")
Tx_reel_court<-scan("Tx_reel_court.txt")
Rdt_actions<-scan("Rdt_actions.txt")

#Dfinition du nombre d'observation dans le fichier en entre
Nobs<-length(Tx_Inflation)
#Dfinition des sries
Tx_Inflation_tplusun=Tx_Inflation[2:Nobs]
Tx_Inflation_t=Tx_Inflation[1:(Nobs-1)]
Tx_reel_court_t=Tx_reel_court[1:(Nobs-1)]
Tx_reel_court_tplusun=Tx_reel_court[2:Nobs]

#Rgression par les MCO
reg_Tx_Inflation=lm(Tx_Inflation_tplusun~Tx_Inflation_t)
reg_Tx_reel_court=lm(Tx_reel_court_tplusun~Tx_reel_court_t)

#Dtermination des rsidus
Resid_Tx_Inflation=resid(reg_Tx_Inflation)/(sd(resid(reg_Tx_Inflation))*sqrt((Nobs-2)/(Nobs-3)))
Resid_Tx_reel_court=resid(reg_Tx_reel_court)/(sd(resid(reg_Tx_reel_court))*sqrt((Nobs-2)/(Nobs-3)))
Resid_Rdt_actions=(Rdt_actions-(mean(Rdt_actions)-(sd(Rdt_actions)^2)/2))/(sd(Rdt_actions)*sqrt(((Nobs+1)-2)/((Nobs+1)-3)))
Resid_Rdt_actions=Resid_Rdt_actions[2:(Nobs)]
Resid_IPC=(Tx_Inflation-(mean(Tx_Inflation)-(sd(Tx_Inflation)^2)/2))/(sd(Tx_Inflation)*sqrt(((Nobs+1)-2)/((Nobs+1)-3)))
Resid_IPC=Resid_IPC[2:(Nobs)]

#Rangement des rsidus dans un tableau
Tableau_ResidBrennanXia=data.frame(Resid_Tx_Inflation,Resid_Tx_reel_court,Resid_Rdt_actions,Resid_IPC)

#Calcul de la matrice de corrlation
Corr_ResidBrennanXia=cor(Tableau_ResidBrennanXia)

##PARTIE 2 : Projection des rsidus en tenant compte des corrlations

#Factorisation de la Cholesky de la matrice de corrlation (matrice triangulaire infrieure)
Mat_Chol=t(chol(Corr_ResidBrennanXia))
#Calcul du nombre d'indice d'actifs dans la matrice de corrlation
Nbcorr=sqrt(length(Corr_ResidBrennanXia))
#Simulation matrice de lois N(0,1)  3 dimensions (nb correlations, nb annees projection, nb simulations)
Mat_norm=array(rnorm(Nbcorr*NS*T),dim=c(Nbcorr,NS,T))
#Initialisation  0 de la matrice  3 dimensions des rsidus
Mat_res=array(0,dim=c(Nbcorr,NS,T),dimnames = list(c("Inflation", "Tx_court","Actions","IPC"),1:NS,1:T))
#Construction de la matrice  3 dimensions avec les rsidus projets
for(i in 1:Nbcorr){
  for(n in 1:NS){
    for(k in 1:T){  
    Mat_res[i,n,k]= sum(Mat_Chol[i,]*Mat_norm[,n,k]) 
    }
  }
}

#Backtesting sur la matrice de corrlation
Corr_Mat_res=array(0,dim=c(NS,Nbcorr,Nbcorr))
for(n in 1:NS){
  Corr_Mat_res[n,,]=cor(t(Mat_res[,n,1:T])) 
}
Test_corr_Mat_res=array(0,dim=c(Nbcorr,Nbcorr),dimnames = list(c("Inflation", "Tx_court","Actions","IPC"),c("Inflation", "Tx_court","Actions","IPC")))
Test_corr_Mat_res[,]=apply(Corr_Mat_res,c(2,3),mean)

##PARTIE 3 : Projection du rendement des actions

# Chargement de la fonction utilise
source(paste(ProjectionFonctionsR,"\\F_Blackscholes.R",sep=""))

#Initialisation des actifs pour le calcul du rendement des actions
RdtAction<-matrix(0,nrow=NS,ncol=(T+1))

#Simulations des actifs pour les actions pour toutes les priodes de projection et rinitialisation des paramtres
for(k in 1:T){
  #Simulations
  RdtAction[,k+1]<-Blackscholes(NS,SigmaAct,MuAct,Mat_res["Actions",,k])
}
plot(((2007+2):(2007+(T))),colMeans(RdtAction[,2:(T)]),type="l",ylim=c(-0.2,0.4),lwd=2,lty=5,pch=5,main="Action (B. et X. / Friggit)",xlab="Anne de projection",ylab="Rendement moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(RdtAction[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(RdtAction[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

##PARTIE 4 : Projection du prix des ZC rels (et projection au pralable des taux d'intrt)

# Chargement de la fonction complmentaire utilise
source(paste(ProjectionFonctionsR,"\\F_Vacisek1.R",sep=""))

#Prsentation des taux courts projets (moyenne des simulations) en vue du calcul du prix des ZC en rel
TauxCourt<-matrix(0,nrow=NS,ncol=(T+1));TauxCourt[,1]=TauxCourtt
for(k in 1:T){
  TauxCourt[,k+1]<-Vacisek1(NS,TauxCourtt,ForceCourt,MoyenneCourt,VolatiliteCourt,Mat_res["Tx_court",,k])
  TauxCourtt<-TauxCourt[,k+1]
}
plot(((2007+2):(2007+(T))),colMeans(TauxCourt[,2:(T)]),type="l",ylim=c(-0.05,0.1),lwd=2,lty=5,pch=5,main="Taux rel court (B. et X. / Friggit)",xlab="Anne de projection",ylab="Taux moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(TauxCourt[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(TauxCourt[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

#Formule de Brennan et Xia du calcul des prix des ZC rels
B=array(0,dim=c(T))
A=array(0,dim=c(T))
ZC_reel=matrix(0,nrow=NS,ncol=(T+1))
TauxCourt_2etoiles=MoyenneCourt+(VolatiliteCourt/ForceCourt)*(Phi_S*Corr_ResidBrennanXia["Resid_Tx_reel_court","Resid_Rdt_actions"]+Phi_r+Phi_I*Corr_ResidBrennanXia["Resid_Tx_reel_court","Resid_Tx_Inflation"])
for(k in 1:T){
  B[k]=((1-exp(-ForceCourt*(M)))/ForceCourt)
  A[k]=(B[k]-M)*TauxCourt_2etoiles-((VolatiliteCourt^2)/(4*(ForceCourt^3)))*(2*ForceCourt*(B[k]-M)+(ForceCourt^2)*((B[k]-M)^2))
  for(n in 1:NS){    
    ZC_reel[n,k+1]=exp(A[k]-B[k]*TauxCourt[n,k+1]) 
  }
}
plot(((2007+2):(2007+(T))),colMeans(ZC_reel[,2:(T)]),type="l",ylim=c(0.9,1.05),lwd=2,lty=5,pch=5,main="ZC rel (Maturit 1 an) (B. et X. / Friggit)",xlab="Anne de projection",ylab="Prix moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(ZC_reel[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(ZC_reel[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

##PARTIE 5 : Projection du prix des ZC nominaux

#Prsentation de l'inflation attendue et ralise projetes (moyenne des simulations) en vue du calcul du prix des ZC en nominal
Inflation<-matrix(0,nrow=NS,ncol=(T+1));Inflation[,1]=It
IPC<-matrix(0,nrow=NS,ncol=(T+1))
for(k in 1:T){
  Inflation[,k+1]<-Vacisek1(NS,It,ForceI,MoyenneI,VolatiliteI,Mat_res["Inflation",,k])
  IPC[,k+1]<-Blackscholes(NS,SigmaIPC,MuIPC,Mat_res["IPC",,k])
  It<-Inflation[,k+1]
}
plot(((2007+2):(2007+(T))),colMeans(Inflation[,2:(T)]),type="l",ylim=c(-0.05,0.1),lwd=2,lty=5,pch=5,main="Inflation attendue (B. et X. / Friggit)",xlab="Anne de projection",ylab="Taux moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(Inflation[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(Inflation[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

plot(((2007+2):(2007+(T))),colMeans(IPC[,2:(T)]),type="l",ylim=c(-0.05,0.1),lwd=2,lty=5,pch=5,main="Inflation ralise (B. et X. / Friggit)",xlab="Anne de projection",ylab="Taux moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(IPC[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(IPC[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)

#Formule de Brennan et Xia du calcul des prix des ZC nominaux
B1=array(0,dim=c(T))
B2=array(0,dim=c(T))
A=array(0,dim=c(T))
ZC_nom=matrix(0,nrow=NS,ncol=(T+1))
TauxCourt_etoile=MoyenneCourt-Lambda_r*VolatiliteCourt/ForceCourt
I_etoile=MoyenneI-Lambda_I*VolatiliteI/ForceI
for(k in 1:T){
  B2[k]=((1-exp(-ForceI*M))/ForceI)
  B1[k]=((1-exp(-ForceCourt*M))/ForceCourt)
  A[k]=(B1[k]-M)*TauxCourt_etoile+(B2[k]-M)*I_etoile-((VolatiliteCourt^2)/(4*(ForceCourt^3)))*(2*ForceCourt*(B1[k]-M)+(ForceCourt^2)*((B1[k]-M)^2))-((VolatiliteI^2)/(4*(ForceI^3)))*(2*ForceI*(B2[k]-M)+(ForceI^2)*((B2[k]-M)^2))+VolatiliteCourt*VolatiliteI*Corr_ResidBrennanXia["Resid_Tx_reel_court","Resid_Tx_Inflation"]/(ForceCourt*ForceI)*(M-B2[k]-B1[k]+(1-exp((ForceCourt+ForceI)*M))/(ForceCourt+ForceI))+(Xi_S*Lambda_S+Xi_r*Lambda_r+Xi_I*Lambda_I+Xi_u*Lambda_u)*M
  for(n in 1:NS){
    ZC_nom[n,k+1]=exp(A[k]-B1[k]*TauxCourt[n,k+1]-B2[k]*Inflation[n,k+1])
  }
}  
plot(((2007+2):(2007+(T))),colMeans(ZC_nom[,2:(T)]),type="l",ylim=c(0.85,1),lwd=2,lty=5,pch=5,main="ZC nominal (Maturit 1 an) (B. et X. / Friggit)",xlab="Anne de projection",ylab="Prix moyen et quantiles  10 % et  90 %")
lines(((2007+2):(2007+(T))),colQuantiles(ZC_nom[,2:(T)],prob=0.9),type="l",lwd=1,lty=5,pch=5)
lines(((2007+2):(2007+(T))),colQuantiles(ZC_nom[,2:(T)],prob=0.1),type="l",lwd=1,lty=5,pch=5)
